# -*- coding: utf-8 -*-
import locale

from django import forms
from django.conf import settings
from django.utils.translation import ugettext_lazy as _

from acomea.areariservata.soap import AcomeaClientRiservataCms
from acomea.areariservata.models import Applicazione


def _get_choices(lista):
    choices = []
    for num, label in enumerate(lista):
        choices.append((num, label))
    return choices


class PosizioneFromForm(forms.Form):
    apps = [(app.codice, app.description) for app in Applicazione.objects.all().filter(status=1).order_by('ordering')]
    tipo_operazione = forms.ChoiceField(
        choices=apps,
        widget=forms.RadioSelect()
    )

    def __init__(self, *args, **kwargs):
        self.session = kwargs.pop('session', None)
        data_operazione = self.session['data_operazione']
        super(PosizioneFromForm, self).__init__(*args, **kwargs)
        choices_positions_from_int = _get_choices(self.session['posizioni_intestatario'])
        choices_positions_from_coint = _get_choices(self.session['posizioni_cointestatario'])

        self.fields['posizione_from_int'] = forms.ChoiceField(
            choices=choices_positions_from_int,
            required=False,
            widget=forms.RadioSelect(),
        )
        self.fields['posizione_from_coint'] = forms.ChoiceField(
            choices=choices_positions_from_coint,
            required=False,
            widget=forms.RadioSelect(),
        )

    def clean(self):
        data = self.cleaned_data

        if not data['posizione_from_int'] and not data['posizione_from_coint']:
            raise forms.ValidationError(_("Selezionare almeno un Rapporto/Contratto"))

        if data['posizione_from_int'] and data['posizione_from_coint']:
            raise forms.ValidationError(_("Selezionare solo un Rapporto/Contratto tra intestatario e cointestatario"))

        if data.get('tipo_operazione'):
            if data['posizione_from_int'] and data['tipo_operazione'] in ('S', 'D'):
                posizione_scelta = self.session['posizioni_intestatario'][int(data['posizione_from_int'])]
                if posizione_scelta['QUOTE'] <= 0:
                    raise forms.ValidationError(_("Operazione non consentita per capienza inesistente"))

            if data['posizione_from_coint'] and data['tipo_operazione'] in ('S', 'D'):
                posizione_scelta = self.session['posizioni_cointestatario'][int(data['posizione_from_coint'])]
                if posizione_scelta['QUOTE'] <= 0:
                    raise forms.ValidationError(_("Operazione non consentita per capienza inesistente"))

        return data


class PosizioneToForm(forms.Form):
    def __init__(self, *args, **kwargs):
        session = kwargs.pop('session', None)
        super(PosizioneToForm, self).__init__(*args, **kwargs)

        self.data_operazione = session['data_operazione']
        self.session = session
        self.posizioni_to = session['posizioni_to']

        if self.data_operazione['tipo_operazione'] == "I":
            choices_positions_to = _get_choices(self.posizioni_to)
            self.fields['posizione_to'] = forms.ChoiceField(
                choices=choices_positions_to,
                widget=forms.RadioSelect(),
                required=False
            )
            self.fields['importo_pic'] = forms.DecimalField(required=False, decimal_places=2, localize=True)
            self.fields['importo_pac'] = forms.DecimalField(required=False, decimal_places=2, localize=True)
            self.fields['numero_rate_versate'] = forms.DecimalField(required=False, decimal_places=2, localize=True)
            self.fields['mifid'] = forms.BooleanField(required=False, initial=False)

        if self.data_operazione['tipo_operazione'] == "S":
            choices_positions_to = _get_choices(self.posizioni_to)
            self.fields['posizione_to'] = forms.ChoiceField(
                choices=choices_positions_to,
                widget=forms.RadioSelect(),
                required=False
            )
            self.fields['tipo_qta_movimento'] = forms.ChoiceField(
                choices=settings.AREARISERVATA_OPERAZIONE_TIPO_QTA_MOVIMENTO_CHOICES,
                widget=forms.RadioSelect(),
                initial="I",
                required=False
            )
            self.fields['importo_pic'] = forms.DecimalField(required=False, decimal_places=2, localize=True)
            self.fields['importo_pac'] = forms.DecimalField(required=False, decimal_places=2, localize=True)
            self.fields['numero_rate_versate'] = forms.DecimalField(required=False, decimal_places=2, localize=True)
            self.fields['quote'] = forms.DecimalField(required=False, decimal_places=3, localize=True)
            self.fields['mifid'] = forms.BooleanField(required=False, initial=False)

        if self.data_operazione['tipo_operazione'] == "D":
            self.fields['tipo_qta_movimento'] = forms.ChoiceField(
                choices=settings.AREARISERVATA_OPERAZIONE_TIPO_QTA_MOVIMENTO_CHOICES,
                widget=forms.RadioSelect(),
                required=False
            )
            self.fields['iban'] = forms.CharField(max_length=27)
            self.fields['importo'] = forms.DecimalField(required=False, decimal_places=2, localize=True)
            self.fields['quote'] = forms.DecimalField(required=False, decimal_places=3, localize=True)

    def can_operate(self, tipo_operazione):
        data = self.cleaned_data
        area_riservata = AcomeaClientRiservataCms()

        mifid = 'N'
        if 'mifid' in data and data['mifid']:
            mifid = 'S'

        # SOTTOSCRIZIONE
        if tipo_operazione == "SOTTOSCRIZIONE":
            indice = int(data['posizione_to'])

            if data['importo_pic']:
                tipo = "IMPORTO"
                valore = data['importo_pic']
                numero_rate_versate = 0
                iban = ''
            if data['importo_pac']:
                tipo = "IMPORTO"
                valore = data['importo_pac']
                numero_rate_versate = data['numero_rate_versate']
                iban = ''

            result = area_riservata.op_online_check_can_operate(
                self.data_operazione['codice_fiscale'], self.data_operazione['codice_cliente'], tipo_operazione,
                self.data_operazione['codice_rapporto_from'], self.data_operazione['codice_deposito_from'],
                self.data_operazione['codice_fondo_from'], self.data_operazione['codice_tipocontratto_from'],
                self.posizioni_to[indice]['RAPPORTO'], self.posizioni_to[indice]['NUMCONTRATTO'],
                self.posizioni_to[indice]['CODFONDO'], self.posizioni_to[indice]['CODTC'], tipo,
                valore, numero_rate_versate, mifid, iban
            )

        # SWITCH
        if tipo_operazione == "SWITCH":

            indice = int(data['posizione_to'])

            if data['tipo_qta_movimento'] == "I":
                if data['importo_pic']:
                    tipo = "IMPORTO"
                    valore = data['importo_pic']
                    numero_rate_versate = 0
                    iban = ''
                if data['importo_pac']:
                    tipo = "IMPORTO"
                    valore = data['importo_pac']
                    numero_rate_versate = data['numero_rate_versate']
                    iban = ''

            elif data['tipo_qta_movimento'] == "Q":
                tipo = "QUOTE"
                valore = data['quote']
                numero_rate_versate = 0
                iban = ''

            elif data['tipo_qta_movimento'] == "T":
                tipo = 'TOTALE'
                valore = 0
                numero_rate_versate = 0
                iban = ''

            result = area_riservata.op_online_check_can_operate(
                self.data_operazione['codice_fiscale'], self.data_operazione['codice_cliente'], tipo_operazione,
                self.data_operazione['codice_rapporto_from'], self.data_operazione['codice_deposito_from'],
                self.data_operazione['codice_fondo_from'], self.data_operazione['codice_tipocontratto_from'],
                self.posizioni_to[indice]['RAPPORTO'], self.posizioni_to[indice]['NUMCONTRATTO'],
                self.posizioni_to[indice]['CODFONDO'], self.posizioni_to[indice]['CODTC'], tipo,
                valore, numero_rate_versate, mifid, iban
            )

        # RIMBORSO
        if tipo_operazione == "RIMBORSO":
            if data['tipo_qta_movimento'] == "I":
                tipo = "IMPORTO"
                valore = data['importo']
                numero_rate_versate = 0
                iban = data['iban']

            elif data['tipo_qta_movimento'] == "Q":
                tipo = "QUOTE"
                valore = data['quote']
                numero_rate_versate = 0
                iban = data['iban']

            elif data['tipo_qta_movimento'] == "T":
                tipo = 'TOTALE'
                valore = 0
                numero_rate_versate = 0
                iban = data['iban']

            result = area_riservata.op_online_check_can_operate(
                self.data_operazione['codice_fiscale'], self.data_operazione['codice_cliente'], tipo_operazione,
                self.data_operazione['codice_rapporto_from'], self.data_operazione['codice_deposito_from'],
                self.data_operazione['codice_fondo_from'], self.data_operazione['codice_tipocontratto_from'],
                self.data_operazione['codice_rapporto_from'], self.data_operazione['codice_deposito_from'],
                self.data_operazione['codice_fondo_from'], self.data_operazione['codice_tipocontratto_from'],
                tipo, valore, numero_rate_versate, mifid, iban
            )

        self.show_mifid = False
        if not result['OK']:
            if result['CheckMIFID'] == "APPROPRIATEZZA":
                self.show_mifid = True
                if not 'mifid' in data or ('mifid' in data and not data['mifid']):
                    raise forms.ValidationError(result['ErrorMessage'])

            if result['CheckMIFID'] == "ADEGUATEZZA":
                raise forms.ValidationError(result['ErrorMessage'])

            if result['CheckMIFID'] == "NONE":
                raise forms.ValidationError(result['ErrorMessage'])

    def clean(self):
        data = self.cleaned_data

        if self.data_operazione['tipo_operazione'] == "I":
            if not 'posizione_to' in data:
                raise forms.ValidationError(_("Seleziona un fondo su cui investire"))
            if not data['posizione_to']:
                raise forms.ValidationError(_("Seleziona un fondo su cui investire"))
            if data['importo_pic'] and data['importo_pac']:
                raise forms.ValidationError(_("Inserire un importo per volta"))
            if not data['importo_pic'] and not data['importo_pac']:
                raise forms.ValidationError(_("Inserire un importo valido"))
            if not data['importo_pac'] and data['numero_rate_versate']:
                raise forms.ValidationError(_("Inserire un importo valido"))
            if data['importo_pac'] and not data['numero_rate_versate']:
                raise forms.ValidationError(_("Inserire un numero rate versate valido"))

            if data['importo_pic'] and data['importo_pic'] <= 0:
                raise forms.ValidationError(_("Inserire un importo maggiore di zero"))
            if data['importo_pac'] and data['importo_pac'] <= 0:
                raise forms.ValidationError(_("Inserire un importo maggiore di zero"))
            if data['numero_rate_versate'] and data['numero_rate_versate'] <= 0:
                raise forms.ValidationError(_("Il numero di rate deve essere maggiore di zero"))

            if 'mifid' in data:
                if data['mifid']:
                    self.session['data_operazione']['mifid'] = True

            self.can_operate('SOTTOSCRIZIONE')

        if self.data_operazione['tipo_operazione'] == "S":
            if not 'posizione_to' in data:
                raise forms.ValidationError(_("Seleziona un fondo su cui switchare"))
            if not data['posizione_to']:
                raise forms.ValidationError(_("Seleziona un fondo su cui switchare"))

            if not 'tipo_qta_movimento' in data:
                raise forms.ValidationError("Selezionare tipo quantità movimento")
            if not data['tipo_qta_movimento']:
                raise forms.ValidationError("Selezionare tipo quantità movimento")

            if data['tipo_qta_movimento'] == "I":
                if not data['importo_pic'] and not data['importo_pac']:
                    raise forms.ValidationError(_("Inserire un importo valido"))
                if data['quote']:
                    raise forms.ValidationError(_("Switch per importo non prevede la presenza di quote"))
            if data['tipo_qta_movimento'] == "Q":
                if not data['quote']:
                    raise forms.ValidationError(_("Inserire un numero di quote valido"))
                if data['importo_pic']:
                    raise forms.ValidationError(_("Switch per quote non prevede la presenza di importi"))
            if data['tipo_qta_movimento'] == "T":
                if data['importo_pic'] or data['quote']:
                    raise forms.ValidationError(_("Switch per totale non prevede la presenza di quote o di importi"))

            if data['importo_pic'] and data['importo_pac']:
                raise forms.ValidationError(_("Inserire un importo per volta"))
            if not 'importo_pac' in data and data['numero_rate_versate']:
                raise forms.ValidationError(_("Inserire un importo valido"))
            if data['importo_pac'] and not 'numero_rate_versate' in data:
                raise forms.ValidationError(_("Inserire un numero rate versate valido"))

            if data['importo_pic'] and data['importo_pic'] <= 0:
                raise forms.ValidationError(_("Inserire un importo maggiore di zero"))
            if data['importo_pac'] and data['importo_pac'] <= 0:
                raise forms.ValidationError(_("Inserire un importo maggiore di zero"))
            if data['numero_rate_versate'] and data['numero_rate_versate'] <= 0:
                raise forms.ValidationError(_("Il numero di rate deve essere maggiore di zero"))
            if data['quote'] and data['quote'] <= 0:
                raise forms.ValidationError(_("Il numero di quote deve essere maggiore di zero"))

            if 'mifid' in data:
                if data['mifid']:
                    self.session['data_operazione']['mifid'] = True

            self.can_operate('SWITCH')

        if self.data_operazione['tipo_operazione'] == "D":
            if not 'tipo_qta_movimento' in data:
                raise forms.ValidationError("Selezionare tipo quantità movimento")
            if not data['tipo_qta_movimento']:
                raise forms.ValidationError("Selezionare tipo quantità movimento")

            if data['tipo_qta_movimento'] == "I" and ('importo' not in data or not data['importo']):
                raise forms.ValidationError(_("Inserire un importo valido"))
            if data['tipo_qta_movimento'] == "Q" and ('quote' not in data or not data['quote']):
                raise forms.ValidationError(_("Inserire un numero di quote valido"))
            if data['importo'] and data['quote']:
                raise forms.ValidationError(_("Selezionare per importo o per quote"))
            if data['tipo_qta_movimento'] == "T" and (data['quote'] or data['importo']):
                raise forms.ValidationError(_("Rimborso totale non prevede la presenza di quote o di importi"))
            if not 'iban' in data:
                raise forms.ValidationError(_("Inserire un IBAN valido"))

            if data['importo'] and data['importo'] <= 0:
                raise forms.ValidationError(_("Inserire un importo maggiore di zero"))
            if data['quote'] and data['quote'] <= 0:
                raise forms.ValidationError(_("Il numero di quote deve essere maggiore di zero"))

            self.can_operate('RIMBORSO')

        return data


class InvestimentoRiepilogoForm(forms.Form):
    def __init__(self, *args, **kwargs):
        session = kwargs.pop('session', None)
        self.data_operazione = session['data_operazione']
        super(InvestimentoRiepilogoForm, self).__init__(*args, **kwargs)

    def can_operate(self):
        data = self.cleaned_data
        area_riservata = AcomeaClientRiservataCms()

        mifid = 'N'
        if self.data_operazione['mifid']:
            mifid = 'S'

        result = area_riservata.op_online_check_can_operate(
            self.data_operazione['codice_fiscale'], self.data_operazione['codice_cliente'], 'SOTTOSCRIZIONE',
            self.data_operazione['codice_rapporto_from'], self.data_operazione['codice_deposito_from'],
            self.data_operazione['codice_fondo_from'], self.data_operazione['codice_tipocontratto_from'],
            self.data_operazione['codice_rapporto_to'], self.data_operazione['codice_deposito_to'],
            self.data_operazione['codice_fondo_to'], self.data_operazione['codice_tipocontratto_to'], 'IMPORTO',
            self.data_operazione['importo'], self.data_operazione['numero_rate_versate'], mifid, ''
        )
        self.show_mifid = False
        if not result['OK']:
            if result['CheckMIFID'] == "APPROPRIATEZZA":
                self.show_mifid = True
                if not 'mifid' in data or ('mifid' in data and not data['mifid']):
                    raise forms.ValidationError(result['ErrorMessage'])
            else:
                raise forms.ValidationError(result['ErrorMessage'])

    def clean(self):
        data = self.cleaned_data
        self.can_operate()
        return data


class SwitchRiepilogoForm(forms.Form):
    def __init__(self, *args, **kwargs):
        session = kwargs.pop('session', None)
        self.data_operazione = session['data_operazione']
        super(SwitchRiepilogoForm, self).__init__(*args, **kwargs)

    def can_operate(self):
        data = self.cleaned_data
        area_riservata = AcomeaClientRiservataCms()

        mifid = 'N'
        if self.data_operazione['mifid']:
            mifid = 'S'

        valore = self.data_operazione['importo']
        if self.data_operazione['tipo_qta_movimento'] == "Q":
            valore = self.data_operazione['quote']

        tipo = None
        if self.data_operazione['tipo_qta_movimento'] == "I":
            tipo = "IMPORTO"
        if self.data_operazione['tipo_qta_movimento'] == "Q":
            tipo = "QUOTE"
        if self.data_operazione['tipo_qta_movimento'] == "T":
            tipo = "TOTALE"
        if not tipo:
            raise forms.ValidationError(_("Operazione scaduta"))

        result = area_riservata.op_online_check_can_operate(
            self.data_operazione['codice_fiscale'], self.data_operazione['codice_cliente'], 'SWITCH',
            self.data_operazione['codice_rapporto_from'], self.data_operazione['codice_deposito_from'],
            self.data_operazione['codice_fondo_from'], self.data_operazione['codice_tipocontratto_from'],
            self.data_operazione['codice_rapporto_to'], self.data_operazione['codice_deposito_to'],
            self.data_operazione['codice_fondo_to'], self.data_operazione['codice_tipocontratto_to'], tipo,
            valore, self.data_operazione['numero_rate_versate'], mifid, ''
        )
        self.show_mifid = False
        if not result['OK']:
            if result['CheckMIFID'] == "APPROPRIATEZZA":
                self.show_mifid = True
                if not 'mifid' in data or ('mifid' in data and not data['mifid']):
                    raise forms.ValidationError(result['ErrorMessage'])
            else:
                raise forms.ValidationError(result['ErrorMessage'])

    def clean(self):
        data = self.cleaned_data
        self.can_operate()
        return data


class DisinvestimentoRiepilogoForm(forms.Form):
    def __init__(self, *args, **kwargs):
        session = kwargs.pop('session', None)
        self.data_operazione = session['data_operazione']
        super(DisinvestimentoRiepilogoForm, self).__init__(*args, **kwargs)

    def can_operate(self):
        data = self.cleaned_data
        area_riservata = AcomeaClientRiservataCms()

        valore = self.data_operazione['importo']
        if self.data_operazione['tipo_qta_movimento'] == "Q":
            valore = self.data_operazione['quote']

        tipo = None
        if self.data_operazione['tipo_qta_movimento'] == "I":
            tipo = "IMPORTO"
        if self.data_operazione['tipo_qta_movimento'] == "Q":
            tipo = "QUOTE"
        if self.data_operazione['tipo_qta_movimento'] == "T":
            tipo = "TOTALE"
        if not tipo:
            raise forms.ValidationError(_("Operazione scaduta"))

        result = area_riservata.op_online_check_can_operate(
            self.data_operazione['codice_fiscale'], self.data_operazione['codice_cliente'], 'SWITCH',
            self.data_operazione['codice_rapporto_from'], self.data_operazione['codice_deposito_from'],
            self.data_operazione['codice_fondo_from'], self.data_operazione['codice_tipocontratto_from'],
            self.data_operazione['codice_rapporto_to'], self.data_operazione['codice_deposito_to'],
            self.data_operazione['codice_fondo_to'], self.data_operazione['codice_tipocontratto_to'], tipo,
            valore, self.data_operazione['numero_rate_versate'], 'N', self.data_operazione['iban']
        )

        if not result['OK']:
            raise forms.ValidationError(result['ErrorMessage'])


class CodiceFiscaleAcomeAForm(forms.Form):
    def __init__(self, *args, **kwargs):
        super(CodiceFiscaleAcomeAForm, self).__init__(*args, **kwargs)
        self.fields['codice_fiscale'] = forms.CharField(
            required=True, max_length=30,
        )
        self.fields['codice_fiscale'].widget.attrs.update({
            'class': 'form-control',
            'placeholder': 'cerca posizione'
        })

    def validate_cliente(self):
        codice_fiscale = self.cleaned_data['codice_fiscale']

        if not codice_fiscale:
            raise forms.ValidationError('Inserire un Codice Fiscale o una Partita IVA valido')

        area_riservata = AcomeaClientRiservataCms()
        result = area_riservata.get_cliente_sgr(codice_fiscale)
        if not result:
            raise forms.ValidationError('Cliente non presente')
        if not result['Found']:
            raise forms.ValidationError('Cliente inesistente')
        if result['IsDead']:
            raise forms.ValidationError('Cliente non attivo')

    def clean_codice_fiscale(self):
        return self.cleaned_data['codice_fiscale'].upper().strip()

    def clean(self):
        data = self.cleaned_data
        if 'codice_fiscale' in data:
            self.validate_cliente()
        else:
            raise forms.ValidationError('Codice fiscale mancante')
        return data
