# -*- coding: utf-8 -*-
import logging

from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.http import HttpResponseRedirect, HttpResponse
from django.template.defaultfilters import floatformat, date
from django.contrib.humanize.templatetags.humanize import intcomma

from jmb.core.views.generic.create_oo_file import create_file
from jmb.core.utils.excel import ExcelReport

from acomea.areariservata.models import Operazione
from acomea.areariservata.utils import clean_session, get_interval
from acomea.areariservata.soap import AcomeaClientRiservataCms

log = logging.getLogger("acomea.areariservata")


@login_required
def eliminazione_operazione_in_corso(request, numero_operazione, codice_fiscale):
    """
    view per eliminare operazioni in corso non ancora valorizzate
    """
    redirect_url = request.META.get('HTTP_REFERER', '/')
    try:
        ar = AcomeaClientRiservataCms()
        result = ar.annulla_operazione_in_corso_web(numero_operazione, codice_fiscale)
        if not result:
            messages.error(request, "Non è possibile effettuare l'operazione richiesta.")
    except:
        messages.error(request, "Non è possibile effettuare l'operazione richiesta.")
    return HttpResponseRedirect(redirect_url)


@login_required
def print_pdf_investimento(request, operazione_id):
    """
   view per stampa pagina investimento
   """
    response_dict = {}
    try:
        operazione = Operazione.objects.get(pk=operazione_id)
        user = request.user
        if not _is_own_operazione(user.profilo, operazione):
            return HttpResponse("Non sei autorizzato per questa azione.")
        importo_operazione = {}
        if operazione.tipo_piano_to == "PIC":
            importo_operazione['messaggio'] = 'Versamento in un unica soluzione'

            importo_operazione['importo'] = _get_comma_separated_str(operazione.importo)
        else:
            importo_operazione['messaggio'] = 'Versamento su piano di accumulo\nNumero rate oggetto del versamento'
            importo_operazione['importo'] = _get_comma_separated_str(operazione.importo)
            importo_operazione['rate_versate'] = _get_comma_separated_str(operazione.numero_rate_versate)
        importo_bonifico = _get_comma_separated_str(operazione.importo)
        iban = "IT46T0343901600000001015007"
        codice_causale = "AR %s" % str(operazione.numero_operazione)
        response_dict.update({
            'operazione': operazione,
            'importo_operazione': importo_operazione,
            'importo_bonifico': importo_bonifico,
            'iban': iban,
            'codice_causale': codice_causale
        })
        return create_file(
            request, file_model='areariservata/cms_plugins/investimento.odt',
            file_name="investimento",
            file_type='pdf', extra_context=response_dict
        )
    except Operazione.DoesNotExist:
        return HttpResponse("Non è possibile effettuare l'operazione richiesta.")


@login_required
def print_pdf_disinvestimento(request, operazione_id):
    """
   view per stampa pagina disinvestimento
   """
    response_dict = {}
    try:
        operazione = Operazione.objects.get(pk=operazione_id)
        user = request.user
        if not _is_own_operazione(user.profilo, operazione):
            return HttpResponse("Non sei autorizzato per questa azione.")
        importo_operazione = {}
        tipo_rimborso = {}
        if operazione.tipo_qta_movimento == "T":
            tipo_rimborso['tipologia'] = "Totale"
        if operazione.tipo_qta_movimento == "Q":
            tipo_rimborso['tipologia'] = "Parziale"
            tipo_rimborso['messaggio'] = "Quote oggetto del rimborso"
            tipo_rimborso['ammontare'] = operazione.quote
        if operazione.tipo_qta_movimento == "I":
            tipo_rimborso['tipologia'] = "Parziale"
            tipo_rimborso['messaggio'] = "Ammontare dell'operazione"
            tipo_rimborso['ammontare'] = u"€ %s " % _get_comma_separated_str(operazione.importo)

        response_dict.update({
            'operazione': operazione,
            'tipo_rimborso': tipo_rimborso,
        })
        return create_file(
            request, file_model='areariservata/cms_plugins/rimborso.odt',
            file_name="rimborso",
            file_type='pdf', extra_context=response_dict
        )
    except Operazione.DoesNotExist:
        return HttpResponse("Non è possibile effettuare l'operazione richiesta.")


@login_required
def print_pdf_switch(request, operazione_id):
    """
   view per stampa pagina switch
   """
    response_dict = {}
    try:
        operazione = Operazione.objects.get(pk=operazione_id)
        user = request.user
        if not _is_own_operazione(user.profilo, operazione):
            return HttpResponse("Non sei autorizzato per questa azione.")
        importo_operazione = {}
        tipo_switch = {}
        if operazione.tipo_qta_movimento == "T":
            tipo_switch['tipologia'] = "Totale"
        if operazione.tipo_qta_movimento == "Q":
            tipo_switch['tipologia'] = "Parziale"
            tipo_switch['messaggio'] = "Quote oggetto dello switch"
            tipo_switch['ammontare'] = operazione.quote
        if operazione.tipo_qta_movimento == "I":
            tipo_switch['tipologia'] = "Parziale"
            tipo_switch['messaggio'] = "Ammontare dell'operazione"
            tipo_switch['ammontare'] = u"€ %s " % _get_comma_separated_str(operazione.importo)

        response_dict.update({
            'operazione': operazione,
            'tipo_switch': tipo_switch,
        })
        return create_file(
            request, file_model='areariservata/cms_plugins/switch.odt',
            file_name="switch",
            file_type='pdf', extra_context=response_dict
        )
    except Operazione.DoesNotExist:
        return HttpResponse("Non è possibile effettuare l'operazione richiesta.")


@login_required
def clean_session_ecommerce(request):
    """
    view per pulizia sessione ecommerce: elimina i valori associati alle chiavi presenti in 'key_list', poi
    effettua il redirect verso 'redirect_url' (se presente in request.POST)
    :param request:
    :return: HttpResponseRedirect
    """
    if request.method == "POST" and "redirect_url" in request.POST:
        redirect_url = request.POST['redirect_url']
        key_list = ['posizioni_intestatario', 'posizioni_cointestatario', 'data_operazione',
                    'posizioni_to', 'result']
        clean_session(request, key_list=key_list)
        return HttpResponseRedirect(redirect_url)
    return HttpResponseRedirect("/")


@login_required()
def print_excel_operazioni_effettuate(request):
    """
    view per stampa foglio excel operazioni effettuate
    """
    filter = request.GET.get('filter' or None)
    date_begin, date_end = get_interval(filter)
    user = request.user
    if user.is_anonymous() or not user.is_active:
        return HttpResponse("Non sei autorizzato per questa azione.")
    try:
        profilo = user.profilo
        rac = user.profilo.rac
    except:
        return HttpResponse("Non sei autorizzato per questa azione.")

    if not profilo.is_cliente and not profilo.is_acomea:
        return HttpResponse("Non sei autorizzato per questa azione.")

    codice_fiscale = None
    operazioni_effettuate = None
    # modifico i dati dell'operazione se l'utente è AcomeA e vuole simulare il cliente
    if profilo.is_acomea and 'codice_fiscale' in request.session:
        codice_fiscale = request.session['codice_fiscale']

    if profilo.is_cliente:
        codice_fiscale = rac.codice_fiscale

    if not codice_fiscale:
        return HttpResponse("Non sei autorizzato per questa azione.")

    area_riservata = AcomeaClientRiservataCms()
    operazioni_intestatario = area_riservata.get_operazioni_effettuate(
        codice_fiscale, 'INTESTATARIO', date_begin, date_end
    )
    operazioni_cointestatario = area_riservata.get_operazioni_effettuate(
        codice_fiscale, 'COINTESTATARIO', date_begin, date_end
    )

    xls = ExcelReport(first_sheet_name="op_effettuate")
    header_row = [
        'Data', 'Cliente', 'Rapporto', 'Contratto', 'Fondo', 'Causale',
        'T.Contr.', 'Quote', 'Imp.Netto'
    ]
    operazioni_dict = {
        "INTESTATARIO": operazioni_intestatario,
        "COINTESTATARIO": operazioni_cointestatario
    }
    for tipo in operazioni_dict.keys():
        if operazioni_dict[tipo]:
            xls.add_row([tipo])
            xls.add_row([])
            xls.add_row(header_row)
            for operazione in operazioni_dict[tipo]:
                values = [
                    date(operazione.get('DataRegistrazione', ''), "d/m/Y"),
                    operazione.get('CodCliente', ''),
                    operazione.get('CodRapporto', ''),
                    operazione.get('NumContratto', ''),
                    operazione.get('DescrFondo', ''),
                    operazione.get('CausaleDescr', ''),
                    operazione.get('TipoContratto', ''),
                    floatformat(operazione.get('Quote', ''), 3),
                    floatformat(operazione.get('ImportoNetto', ''), 2)
                ]
                xls.add_row(values)
            xls.add_row([])
    cliente = area_riservata.get_cliente(codice_fiscale)
    response = HttpResponse(mimetype='application/vnd.ms-excel')
    filename = cliente.get("Cognome", "") + "_" + cliente.get("Nome", "") + "_operazioni-effettuate"
    if filter:
        filename += "_" + filter
    response['Content-Disposition'] = 'attachment; filename=%s.xls' % filename
    response.write(xls.write_report())
    return response


@login_required()
def print_excel_operazioni_in_corso(request):
    """
    view per stampa foglio excel operazioni in corso
    """
    user = request.user
    if user.is_anonymous() or not user.is_active:
        return HttpResponse("Non sei autorizzato per questa azione.")
    try:
        profilo = user.profilo
        rac = user.profilo.rac
    except:
        return HttpResponse("Non sei autorizzato per questa azione.")

    if not profilo.is_cliente and not profilo.is_acomea:
        return HttpResponse("Non sei autorizzato per questa azione.")

    codice_fiscale = None
    operazioni_in_corso = None
    # modifico i dati dell'operazione se l'utente è AcomeA e vuole simulare il cliente
    if profilo.is_acomea and 'codice_fiscale' in request.session:
        codice_fiscale = request.session['codice_fiscale']

    if profilo.is_cliente:
        codice_fiscale = rac.codice_fiscale

    if not codice_fiscale:
        return HttpResponse("Non sei autorizzato per questa azione.")

    area_riservata = AcomeaClientRiservataCms()
    operazioni_internet = area_riservata.get_operazioni_in_corso_web(codice_fiscale)
    operazioni_as400 = area_riservata.get_operazioni_in_corso_contra(codice_fiscale)

    xls = ExcelReport(first_sheet_name="op_in_corso")
    operazioni_dict = {
        "OPERAZIONI INTERNET": operazioni_internet,
        "OPERAZIONI AS400 DA VALORIZZARE": operazioni_as400,
    }

    #---===   OPERAZIONI INTERNET   ===---#
    if operazioni_dict["OPERAZIONI INTERNET"]:
        xls.add_row(["Operazioni in corso pervenute via internet"])
        xls.add_row([])
        xls.add_row([
            'Causale bonifico', 'Cliente', 'Rapporto', 'Contratto', 'Fondo', 'Data/Ora', 'Causale',
            'Quote', 'I. Lordo', 'F.Switch'
        ])
        for operazione in operazioni_dict["OPERAZIONI INTERNET"]:
            nop = str(operazione.get('Nop', ''))
            values = [
                'AR {0}'.format(nop) if nop else '',
                operazione.get('CodCliente', ''),
                operazione.get('CodRapp', ''),
                operazione.get('CodDeposito', ''),
                operazione.get('DescrFondo', ''),
                date(operazione.get('Data', ''), "d/m/Y"),
                operazione.get('CausaleDescr', ''),
                floatformat(operazione.get('Quote', ''), 3),
                floatformat(operazione.get('Importo', ''), 2),
                operazione.get('SwitchDescrFon', ''),
            ]
            xls.add_row(values)
        xls.add_row([])

    #---===   OPERAZIONI CONTRA AS400   ===---#
    if operazioni_dict["OPERAZIONI AS400 DA VALORIZZARE"]:
        xls.add_row(["Operazioni in corso ancora da valorizzare"])
        xls.add_row([])
        xls.add_row([
            '', 'Cliente', 'Rapporto', 'Contratto', 'Fondo', 'Data/Ora',
            'Causale.', 'Quote', 'I.Lordo', 'F.Switch'
        ])
        for operazione in operazioni_dict["OPERAZIONI AS400 DA VALORIZZARE"]:
            values = [
                '',
                operazione.get('CodCliente', ''),
                operazione.get('CodRapporto', ''),
                operazione.get('NumContratto', ''),
                operazione.get('DescrFondo', ''),
                date(operazione.get('Data', ''), "d/m/Y"),
                operazione.get('CausaleDescr', ''),
                floatformat(operazione.get('Quote', ''), 3),
                floatformat(operazione.get('ImportoLordo', ''), 2),
                operazione.get('SwitchDescrFon', ''),
            ]
            xls.add_row(values)
        xls.add_row([])

    cliente = area_riservata.get_cliente(codice_fiscale)
    response = HttpResponse(mimetype='application/vnd.ms-excel')
    filename = cliente.get("Cognome", "") + "_" + cliente.get("Nome", "") + "_operazioni-in-corso"
    response['Content-Disposition'] = 'attachment; filename=%s.xls' % filename
    response.write(xls.write_report())
    return response


def _get_comma_separated_str(decimal_val):
    """
    workaround per formattare correttamente le stringhe/unicode che rappresentano numeri.
    Separatori:
    - "," : parte decimale
    - "." : migliaia
    """

    from django.template.defaultfilters import floatformat
    from django.contrib.humanize.templatetags.humanize import intcomma

    float_val = floatformat(decimal_val, 2)
    float_val = float_val.replace(',', '.')
    unicode_val = intcomma(float_val)
    values = unicode_val.split('.')
    int_part = values[0].replace(',', '.')
    dec_part = values[1]

    return ",".join([int_part, dec_part])


def _is_own_operazione(profilo, operazione):
    """
    Controlla che l'operazione sia affettivamente associata a quel profilo.

    """
    if profilo.is_acomea:
        return True
    return profilo.id == operazione.profilo.id